﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using Telerik.Windows.Controls;
using Telerik.Windows.Controls.TreeView;
using System.Collections;
using Telerik.Windows.Controls.DragDrop;
using Telerik.Windows.Controls.GridView;
using Telerik.Windows.Controls.TreeListView;

namespace WpfApplication64
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            dragTree.ItemsSource = new ObservableCollection<Vehicle>() { new Vehicle("Cars", true), new Vehicle("Vans", true) };
            dropTree.ItemsSource = new ObservableCollection<Vehicle>();

            RadDragAndDropManager.SetAllowDrop(dropTree, true);
            RadDragAndDropManager.AddDropQueryHandler(dropTree, OnDropQuery);
            RadDragAndDropManager.AddDropInfoHandler(dropTree, OnDropInfo);
            RadDragAndDropManager.AddDragQueryHandler(dragTree, OnDragQuery);
            RadDragAndDropManager.AddDragInfoHandler(dragTree, OnDragInfo);
        }

        private void OnDropQuery(object sender, DragDropQueryEventArgs e)
        {
            if (e.Options.Source is GridViewHeaderCell)
            {
                return;
            }

            ICollection draggedItems = e.Options.Payload as ICollection;
            bool result = draggedItems.Cast<object>().All((object item) => item is Vehicle);
            e.QueryResult = result;
            e.Handled = true;
        }

        private void OnDropInfo(object sender, DragDropEventArgs e)
        {
            RadTreeListView tree = new RadTreeListView();
            if (e.Options.Destination as RadTreeListView != null)
                tree = e.Options.Destination as RadTreeListView;
            else if (e.Options.Destination as TreeListViewRow != null)
                tree = (e.Options.Destination as TreeListViewRow).ParentOfType<RadTreeListView>();

            IList draggedItems = e.Options.Payload as IList;

            TreeViewDragCue cue = e.Options.DragCue as TreeViewDragCue;

            if (e.Options.Status == DragStatus.DropPossible)
            {
                cue.IsDropPossible = true;
            }
            else if (e.Options.Status == DragStatus.DropImpossible)
            {
                cue.IsDropPossible = false;
            }
            else if (e.Options.Status == DragStatus.DropComplete)
            {
                IList items = tree.ItemsSource as IList;
                if (e.Options.Destination as TreeListViewRow != null)
                {
                    TreeListViewRow destinationRow = e.Options.Destination as TreeListViewRow;
                    TreeListViewDropPosition dropPosition = (TreeListViewDropPosition)destinationRow.GetValue(RadTreeListView.DropPositionProperty);
                    if (dropPosition == TreeListViewDropPosition.After)
                    {
                        int index = (destinationRow.ParentOfType<RadTreeListView>().ItemContainerGenerator.IndexFromContainer(destinationRow));
                        ObservableCollection<Vehicle> collectionToAddIn = GetParentCollection(destinationRow.ParentOfType<RadTreeListView>().ItemsSource as ObservableCollection<Vehicle>, destinationRow.Item as Vehicle);
                        foreach (object draggedItem in draggedItems)
                        {
                            collectionToAddIn.Insert(index + 1, draggedItem as Vehicle);
                        }
                    }
                    else if (dropPosition == TreeListViewDropPosition.Before)
                    {
                        int index = (destinationRow.ParentOfType<RadTreeListView>().ItemContainerGenerator.IndexFromContainer(destinationRow));
                        ObservableCollection<Vehicle> collectionToAddIn = GetParentCollection(destinationRow.ParentOfType<RadTreeListView>().ItemsSource as ObservableCollection<Vehicle>, destinationRow.Item as Vehicle);
                        foreach (object draggedItem in draggedItems)
                        {
                            collectionToAddIn.Insert(index, draggedItem as Vehicle);
                        }
                    }
                    else if (dropPosition == TreeListViewDropPosition.Inside)
                    {

                        foreach (object draggedItem in draggedItems)
                        {
                            ((destinationRow.Item as Vehicle).Items as ObservableCollection<Vehicle>).Add(draggedItem as Vehicle);
                        }
                    }
                }
                else if (e.Options.Destination as RadTreeListView != null)
                {
                    foreach (object draggedItem in draggedItems)
                    {
                        items.Add(draggedItem);
                    }
                }
            }
        }

        private ObservableCollection<Vehicle> GetParentCollection(ObservableCollection<Vehicle> vehicles, Vehicle vehicle)
        {
            if (vehicles.Contains(vehicle))
            {
                return vehicles;
            }
            else
            {
                foreach (var vhcl in vehicles)
                {
                    var collection = GetParentCollection(vhcl.Items, vehicle);
                    if (collection != null)
                        return collection;
                }
            }

            return null;
        }

        private void OnDragQuery(object sender, DragDropQueryEventArgs e)
        {
            if (e.Options.Source is GridViewHeaderCell)
            {
                return;
            }

            RadTreeListView tree = sender as RadTreeListView;
            if (tree != null)
            {
                IList selectedItems = tree.SelectedItems.ToList();
                e.Options.Payload = selectedItems;
            }

            e.QueryResult = true;
            e.Handled = true;
        }

        private void OnDragInfo(object sender, DragDropEventArgs e)
        {
            if (e.Options.Source is GridViewHeaderCell)
            {
                return;
            }

            IEnumerable draggedItems = e.Options.Payload as IEnumerable;

            if (e.Options.Status == DragStatus.DragInProgress)
            {
                TreeViewDragCue cue = new TreeViewDragCue();
                cue.ItemsSource = draggedItems;
                e.Options.DragCue = cue;
            }
            else if (e.Options.Status == DragStatus.DragComplete)
            {
                ObservableCollection<Vehicle> source = this.dragTree.ItemsSource as ObservableCollection<Vehicle>;
                foreach (object draggedItem in draggedItems)
                {
                    RemoveVehicle(source, draggedItem as Vehicle);
                }
            }
        }

        private void RemoveVehicle(ObservableCollection<Vehicle> vehicles, Vehicle vehicle)
        {
            if (!vehicles.Remove(vehicle))
            {
                foreach (var vhcl in vehicles)
                {
                    RemoveVehicle(vhcl.Items, vehicle);
                }
            }
        }
    }

    public class Vehicle
    {
        private static Random rnd = new Random();

        public Vehicle(string type, bool flag)
        {
            this.Type = type;
            this.ID = rnd.Next();
            this.Items = new ObservableCollection<Vehicle>();
            if (flag)
                for (int i = 0; i < 10; i++)
                {
                    this.Items.Add(new Vehicle(type + "_" + i.ToString()));
                }
        }

        public Vehicle(string type)
        {
            this.Type = type;
            this.ID = rnd.Next();
            this.Items = new ObservableCollection<Vehicle>();
        }

        public Vehicle()
        {

        }

        public string Type
        {
            get;
            set;
        }

        public int ID
        {
            get;
            set;
        }

        public ObservableCollection<Vehicle> Items
        {
            get;
            set;
        }
    }
}
